;*******************************************************************************
;
;	Filename:		MSMVx2.asm
;	Date:				19 Jun 2017
;	File Version:	1.0
;		1.0 - original code
;	Author:			Larry Cicchinelli
;	Description:
;		This program uses a PIC 12F1572 to emulate two 555 monostable one-shots.

MSMV_1		equ	1			; used only when debugging
MSMV_2		equ	2

DEBUG			equ	0			; define to debug one of the MSMVs, 0 = no debug
;	The only difference when debugging between MSMV 1 and MSMV 2 is which timer
;		is used - the I/O pins are the same
;
;	Definitions that affect the operation of the one-shots
;		T0PRS						set MSMV 1 resolution
;		T2PRS, T2PST			set MSMV 2 resolution
;		MSMV_1_EDGE_REG		set MSMV 1 trigger edge
;		MSMV_2_EDGE_REG		set MSMV 2 trigger edge
;		MSMV_1_RETRIG			set to 1 to enable MSMV 1 retriggering
;		MSMV_2_RETRIG			set to 1 to enable MSMV 2 retriggering
;
; Software and programming notes:
;	MSMV 1 uses Timer 0
;	MSMV 2 uses Timer 2
;		both are 8 bits
;
;	The trigger software uses the Interrupt-On-Change feature to set the
;		appropriate IOCAF bit when the trigger is detected.  This is done so that
;		the trigger will be detected even if it is a very short pulse.
;		The output pulse is started in the ISR when the trigger is detected.
;	The triggering edge may be changed for either or both MSMVs by changing
;		the definition of MSMV_1_EDGE_REG and/or MSMV_2_EDGE_REG
;
;	The sensitivity to the LSB of the A/D readings my be eliminated in either of
;		two ways
;		1) clearing bit 0 on every reading - this will multiply the resolution by a
;			a factor of two
;		2) shift the A/D reading by one bit to the right - this will not change
;			the resolution but will reduce the full scale value for a factor of two.
;
;	Timer overflows are counted in the ISR.  When the number of overflows that
;		correspond to the A/D reading have occured, the output pulse is terminated.
;		The A/D readings used for the PW are copied to the operating RAM only when
;		a trigger occurs.
;	The base clock for both timers is Fosc/4 = 8MHz, period = 125ns.
;	Timer 0 resolution (time between interrupts) is determined by the definition
;		of T0PRS to one of the T0PRS_x values.  The minimum value is 32us,
;		the maximum value is 8.192 ms.
;	Timer 2 resolution is determined by two definitions:
;		T2PRS to one of four T2PRS_x values: 1, 4, 16 or 64
;		T2PST to one of 16 vlaues: 1..16
;	The range of Timer 2 resolutions is: 32us to 32.768ms
;	Other values of resolution may be obtained by setting TMRx to a non-0 value.
;		For instance, if 100us resolution is desired for MSMV 1:
;			set T0PRS	equ	T0PRS_4		; normally 128usec resolution
;			load TMR0 with 255. - (2*28.) = 199.
;
;	CPU clock is 32MHz
;	The A/D sample clock is 1MHz
;	Tab = 3 spaces
;
;		PIC12F1572 I/O for "normal" operation
;			pin 1: VDD
;			pin 2: RA5				MSMV 1 Output
;			pin 3: RA4, AN3		MSMV 1 PW
;			pin 4: RA3, ~MCLR		MSMV 1 Trigger
;			pin 5: RA2, AN2		MSMV 2 Trigger
;			pin 6: RA1, AN1		MSMV 2 PW
;			pin 7: RA0, AN0		MSMV 2 Output
;			pin 8: VSS
;
;		PIC12F1572 I/O for debugging
;			pin 1: VDD
;			pin 2: RA5				MSMV Output
;			pin 3: RA4, AN3		MSMV PW
;			pin 4: RA3,			~MCLR
;			pin 5: RA2, AN2		MSMV Trigger
;			pin 6: RA1, AN1	ICSPCLK
;			pin 7: RA0, AN0	ICSPDAT
;			pin 8: VSS
;
; Definitions used by the program which are associated with specific I/O pins:
;	AI_PW_1		- MSMV 1 analog voltage which defines the pulse width
;	AI_PW_2		- MSMV 2 analog voltage which defines the pulse width
;	DO_PULSE_1	- MSMV 1 output pulse
;	DO_PULSE_2	- MSMV 2 output pulse
;	DI_TRIG_1	- MSMV 1 Trigger
;	DI_TRIG_2	- MSMV 2 Trigger
;
;*******************************************************************************

;*******************************************************************************

		#include p12F1572.inc
#if DEBUG == 0
	__CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
 else	; the only difference is _MCLRE_ON vs _MCLRE_OFF
	__CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF
#endif
	__CONFIG _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_ON & _BORV_LO & _LPBOREN_OFF & _LVP_OFF

;*******************************************************************************
; I/O signal definitions
;*******************************************************************************

#if DEBUG == 0

AI_PW_1		equ	RA4
ADCH_PW_1	equ	( 3<<CHS0 ) | (1<<ADON )	; RA4 is AN3
DO_PULSE_1	equ	RA5
DI_TRIG_1	equ	RA3

AI_PW_2		equ	RA1
ADCH_PW_2	equ	( 1<<CHS0 ) | (1<<ADON )
DO_PULSE_2	equ	RA0
DI_TRIG_2	equ	RA2

#else

AI_PW_1		equ	RA4
ADCH_PW_1	equ	( 3<<CHS0 ) | (1<<ADON )	; RA4 is AN3
DO_PULSE_1	equ	RA5
DI_TRIG_1	equ	RA2

AI_PW_2		equ	AI_PW_1
ADCH_PW_2	equ	ADCH_PW_1
DO_PULSE_2	equ	DO_PULSE_1
DI_TRIG_2	equ	DI_TRIG_1

#endif

AI_MASK		equ	(1<<AI_PW_1) | (1<<AI_PW_2)
DI_MASK		equ	(1<<DI_TRIG_1) | (1<<DI_TRIG_2)

TRISA_VAL	equ	AI_MASK | DI_MASK
ANSEL_VAL	equ	AI_MASK
	
; Resolution values are based on CPU clock = 32MHz
; The  resolution values may be increased by multiplying the A/D readings
; by up to 64 (in order to keep within a 16 bit value).

; Timer 0 prescale definitions, used by T0PRS in OPTION_REG
T0PRS_1		equ	(1<<PSA)			; 32us	PSA = 1 = no prescaler
T0PRS_2		equ	0 | (0<<PSA)	; 64		PSA = 0 = use prescaler
T0PRS_4		equ	1 | (0<<PSA)	; 128
T0PRS_8		equ	2 | (0<<PSA)	; 256
T0PRS_16		equ	3 | (0<<PSA)	; 512
T0PRS_32		equ	4 | (0<<PSA)	; 1.024ms
T0PRS_64		equ	5 | (0<<PSA)	; 2.048
T0PRS_128	equ	6 | (0<<PSA)	; 4.096
T0PRS_256	equ	7 | (0<<PSA)	; 8.192

; Timer 2 prescale definitions, used by T2PRS in T2CON
T2PRS_1		equ	0						; basic resolution = 32us
T2PRS_4		equ	( 1<< T2CKPS0 )	; = 128us
T2PRS_16		equ	( 2<< T2CKPS0 )	; = 512us
T2PRS_64		equ	( 3<< T2CKPS0 )	; = 2.048ms

; Timer 2 postcale definitions, used by T2PST in T2CON
T2PST_1		equ	0
T2PST_2		equ	( 1<<T2OUTPS0 )
T2PST_3		equ	( 2<<T2OUTPS0 )
T2PST_4		equ	( 3<<T2OUTPS0 )
T2PST_5		equ	( 4<<T2OUTPS0 )
T2PST_6		equ	( 5<<T2OUTPS0 )
T2PST_7		equ	( 6<<T2OUTPS0 )
T2PST_8		equ	( 7<<T2OUTPS0 )
T2PST_9		equ	( 8<<T2OUTPS0 )
T2PST_10		equ	( 9<<T2OUTPS0 )
T2PST_11		equ	( .10<<T2OUTPS0 )
T2PST_12		equ	( .11<<T2OUTPS0 )
T2PST_13		equ	( .12<<T2OUTPS0 )
T2PST_14		equ	( .13<<T2OUTPS0 )
T2PST_15		equ	( .14<<T2OUTPS0 )
T2PST_16		equ	( .15<<T2OUTPS0 )
		
; A/D conversion clock definitions
ADCSv2		equ	(0<<ADCS0)		; Fosc/2
ADCSv8		equ	(1<<ADCS0)		; Fosc/8
ADCSv32		equ	(2<<ADCS0)		; Fosc/32
ADCSvFRC		equ	(3<<ADCS0)		; FRC - abt 500KHz
ADCSv4		equ	(4<<ADCS0)		; Fosc/4
ADCSv16		equ	(5<<ADCS0)		; Fosc/16
ADCSv64		equ	(6<<ADCS0)		; Fosc/64
ADCS_FRC		equ	(7<<ADCS0)		; FRC - abt 500KHz


;*******************************************************************************
; user definitions
;*******************************************************************************

T0PRS				equ	T0PRS_1		; select MSMV 1 resolution

T2PRS				equ	T2PRS_64		; select MSMV 2
T2PST				equ	T2PST_1		;		resolution

MSMV_1_EDGE_REG equ	IOCAN			; IOCAP = trigger on positive edge of input
MSMV_2_EDGE_REG equ	IOCAN			; IOCAN = trigger on negative edge of input

MSMV_1_RETRIG	equ	0				; 1 = enable retriggering
MSMV_2_RETRIG	equ	0				; 1 = enable retriggering

;*******************************************************************************
; Common RAM allocation: 16 bytes
;*******************************************************************************
Access_RAM		udata_shr	0x70		; 16 bytes starting at address 0x70
PW_1				res	2					; double buffer so that updating with A/D
PW_1A				res	2
PW_2				res	2					;		does not affect the ISR timeout
PW_2A				res	2

; Since the total number of bytes of RAM required <= 16, the program does not
; need to make use of banked RAM


	org	0
	goto	Main

	org	0x04								; interrupt vector
; This ISR is executed whenever any of the following occurs:
;	1) Timer 0 times out
;	2) Timer 1 times out
;	3) Trigger edge for OS 1
;	4) Trigger edge for OS 2

#if DEBUG == 0 || DEBUG == MSMV_1
; test for Timer 0 interrupt
	btfss		INTCON, TMR0IF				; skip next if Timer 0 interrupt
	goto		ISR.TMR0.Ex
; here if Timer 0 interrupt
	bcf		INTCON, TMR0IF				; clear interrupt flag
; update PW counter
	movlw		1
	subwf		PW_1A, f						; update low byte
	clrf		WREG
	subwfb	PW_1A+1, f					; update high byte
	movfw		PW_1A							; set up to
	iorwf		PW_1A+1, w					;		test for done
	btfss		STATUS, Z					; skip next if done PW
	goto		ISR.TMR0.Ex
; here if done  MSMV 1 pulse
	BANKSEL	LATA
	bcf		LATA, DO_PULSE_1			; turn off output pulse
	bcf		INTCON, TMR0IE				; disable Timer 0 interrupts
	; Timer 0 interrupts are disabled since there is no way to disable the Timer 
	BANKSEL	IOCAP
	bsf		MSMV_1_EDGE_REG, DI_TRIG_1	; enable MSMV 1 edge detect
ISR.TMR0.Ex

; test for MSMV 1 trigger
	BANKSEL	IOCAF
	btfss		IOCAF, DI_TRIG_1			; skip next if MSMV 1 trigger
	goto		ISR.TRIG1.Ex
; here if MSMV 1 trigger
	bcf		IOCAF, DI_TRIG_1			; clear the interrupt flag
#if MSMV_1_RETRIG == 0
	bcf		MSMV_1_EDGE_REG, DI_TRIG_1	; disable MSMV 1 edge detect
#endif
	BANKSEL	LATA
	bsf		LATA, DO_PULSE_1			; turn on output pulse
	BANKSEL	TMR0
	clrf		TMR0							; clear the Timer 0 counter
	movfw		PW_1							; get low byte of PW
	movwf		PW_1A							; save in working register
	movfw		PW_1+1						; get high byte of PW
	movwf		PW_1A+1						; save in working register
	bsf		INTCON, TMR0IE				; enable Timer 0 interrupts
ISR.TRIG1.Ex
#endif	; DEBUG == 0 || DEBUG == MSMV_1

#if DEBUG == 0 || DEBUG == MSMV_2
; test for Timer 2 interrupt
	BANKSEL	PIR1
	btfss		PIR1, TMR2IF				; skip next if Timer 2 interrupt
	goto		ISR.TMR2.Ex
; here if Timer 2 interrupt
	bcf		PIR1, TMR2IF				; clear interrupt flag
; update PW counter
	movlw		1
	subwf		PW_2A, f						; update low byte
	clrf		WREG
	subwfb	PW_2A+1, f					; update high byte
	movfw		PW_2A							; set up to
	iorwf		PW_2A+1, w					;		test for done
	btfss		STATUS, Z					; skip next if done PW
	goto		ISR.TMR2.Ex
; here if done  MSMV 2 pulse
	BANKSEL	LATA
	bcf		LATA, DO_PULSE_2			; turn off output pulse
	; Timer 2 interrupts are left enabled since the Timer itself is disabled
	BANKSEL	T2CON
	bcf		T2CON, TMR2ON				; disable Timer 2
	BANKSEL	IOCAP
	bsf		MSMV_2_EDGE_REG, DI_TRIG_2	; enable MSMV 2 edge detect
ISR.TMR2.Ex

; test for MSMV 2 trigger
	BANKSEL	IOCAF
	btfss		IOCAF, DI_TRIG_2			; skip next if MSMV 2 trigger
	goto		ISR.TRIG2.Ex
; here if MSMV 2 trigger
	bcf		IOCAF, DI_TRIG_2			; clear the interrupt flag
#if MSMV_2_RETRIG == 0
	bcf		MSMV_2_EDGE_REG, DI_TRIG_2	; disable MSMV 2 edge detect
#endif
	BANKSEL	LATA
	bsf		LATA, DO_PULSE_2			; turn on output pulse
	movfw		PW_2							; get low byte of PW
	movwf		PW_2A							; save in working register
	movfw		PW_2+1						; get high byte of PW
	movwf		PW_2A+1						; save in working register
	BANKSEL	TMR2
	clrf		TMR2							; clear the counter, pre and postscalers
	bsf		T2CON, TMR2ON				; enable Timer 2
ISR.TRIG2.Ex
#endif	; DEBUG == 0 || DEBUG == MSMV_2

	retfie


;*******************************************************************************
; MAIN PROGRAM
;*******************************************************************************

MAIN_PROG CODE                      ; let linker place main program

Main
	call		init							; initialize the system hardware
	clrf		FSR0H							; always points to common RAM

	call		ReadPW						; read both pulse widths
; set up edge detect
	BANKSEL	IOCAP
	bsf		MSMV_1_EDGE_REG, DI_TRIG_1	; enable MSMV_1 edge detect
	bsf		MSMV_2_EDGE_REG, DI_TRIG_2	; enable MSMV_2 edge detect

; set Timer 2 registers
	BANKSEL	TMR2
	movlw		0xFF							; set the match register
	movwf		PR2							;		to 255
	movlw		T2PRS | T2PST				; set pre and post scale values
	movwf		T2CON							; and Timer2 off
	BANKSEL	PIE1							; enable Timer2 interrupts
	bsf		PIE1, TMR2IE

; enable appropriate interrupts
#if DEBUG == MSMV_2
	movlw		(1<<GIE) | (1<<PEIE) | (1<<IOCIE)
#else
	movlw		(1<<GIE) | (1<<PEIE) | (1<<TMR0IE) | (1<<IOCIE)
#endif
	movwf		INTCON						; enable interrupts

Main.100
	call		ReadPW						; read both pulse widths
	goto		Main.100


;===============================================================================
;									Main Operating Functions
;===============================================================================

ReadPW
; read both PW values
; read MSMV 1
	movlw		PW_1
	movwf		FSR0L
	movlw		ADCH_PW_1
	call		AD.ReadW
; read MSMV 2
	movlw		PW_2
	movwf		FSR0L
	movlw		ADCH_PW_2
	call		AD.ReadW
	return


AD.ReadW
; read the 10 bit A/D
; enter with:
;	w = A/D channel in appropriate bits and ADON bit set
;	FSR0 = address to put result
; exit with:
;	10 bit value in ADvalue and ADvalue+1
; about 28 instruction cycles + about 12usec with 32MHz clock
;	total of about 16usec with 32MHz clock

; set the channel and keep A/D on
	BANKSEL	ADCON0
	movwf		ADCON0
; delay for acquisition time - 20 instruction sycles
	movlw		.7
AD.ReadW.05
	decfsz	WREG, f						; 1 update count, skip next if done
	goto		AD.ReadW.05					; 2
	bsf		ADCON0, ADGO				; start the conversion
; wait for reading to complete
AD.ReadW.10
	btfsc		ADCON0, ADGO				; skip next if done conversion
	goto		AD.ReadW.10					; wait for done
; test for 0
	movfw		ADRESL
	addwf		ADRESH, w
	btfsc		STATUS, Z					; skip next if value not 0
	bsf		ADRESL, 0					; change value of 0 to 1
; store the reading
	bcf		INTCON, IOCIE				; disable interrupt-on-change to ensure ISR
												;		does not use partially updated value
	movfw		ADRESL						; save the low byte
	movwi		FSR0++
	movfw		ADRESH						; save the high byte
	movwi		FSR0--
	bsf		INTCON, IOCIE				; enable interrupt-on-change
; interrupts are disabled for 5 instruction times == 625ns @32MHz clock

	return
; AD.ReadW


Set32MHz
; set FOSC to 32MHz using 8MHz clock and PLL
	BANKSEL	OSCCON
	movlw		(1<<SPLLEN) | (b'1110'<<IRCF0) | (b'00'<<SCS0)
	movwf		OSCCON			; enable PLL, IntOsc=8Mhz, clock determined by CONFIG
									; *** SCS must be '00' to allow PLL operation ***
Set32MHz.10
	btfss		OSCSTAT, HFIOFL			; skip next if HF osc ready
	goto		Set32MHz.10
Set32MHz.20
	btfss		OSCSTAT, PLLR				; skip next if PLL ready
	goto		Set32MHz.20
	return


;===============================================================================
;									Initialize Hardware
;===============================================================================

init	; initialize the system hardware

	call		Set32MHz						; set FOSC to 32MHz
; Port A
	BANKSEL	PORTA
	clrf		PORTA
	BANKSEL	TRISA
	movlw		TRISA_VAL
	movwf		TRISA
	BANKSEL	ANSELA
	movlw		ANSEL_VAL
	movwf		ANSELA
	BANKSEL	INLVLA
	movlw		0xFF							; set up to
	movwf		INLVLA						;		make all inputs Schmitt Trigger CMOS
	BANKSEL	WPUA
	clrf		WPUA							; disable all weak pullups

; A/D convertor
	BANKSEL	ADCON0
	movlw		(1<<ADON)					; A/D on, used later to select A/D channel
	movwf		ADCON0
	movlw		(1<<ADFM) | ADCSv32 | (0<<ADPREF0)
	movwf		ADCON1						; right justify data, clk=Fosc/32, VRPOS=VDD
	clrf		ADCON2						; use ADCON0(1<<ADGO) as trigger

; Timer 0
	BANKSEL	OPTION_REG
	movlw		(0<<TMR0CS) | (T0PRS) ; clock = Fosc/4
	movwf		OPTION_REG

; Interrupts
	clrf		INTCON						; clear global interrupt enable
	BANKSEL	PIE1
	clrf		PIE1							; clear peripheral interrupt enables
	BANKSEL	PORTA

	return
; init

	END
